package Socialtext::Resting::Utils;
use strict;
use warnings;

=head1 NAME

Socialtext::Resting::Utils - Utilities for Socialtext REST APIs

=cut

our $VERSION = '0.07';

=head1 SYNOPSIS

Socialtext::Resting::Utils does not contain any code.

=head1 AUTHOR

Luke Closs, C<< <luke.closs at socialtext.com> >>

=head1 COPYRIGHT & LICENSE

Copyright 2006 Luke Closs, all rights reserved.

This program is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.

=cut

1;
